@@ -14,7 +14,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/acg.c,v 1.16 2007/10/26 21:00:37 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/acg.c,v 1.17 2007/12/08 19:24:06 dirk Exp $ */
 
 #include <ctype.h>
 #include <errno.h>
@@ -319,7 +319,10 @@ int acg_setup_dual_2_2(int fd)
     }
 
   char *str="setup for rfdump";
-  return acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+  acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+
+  rfidReset(fd);
+  return 0;
 }
 
 /***************************************************************************/
@@ -569,7 +572,10 @@ int acg_setup_multiiso_1_0(int fd)
     }
 
   char *str="setup for rfdump";
-  return acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+  acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+
+  rfidReset(fd);
+  return 0;
 }
 
 /***************************************************************************/
@@ -743,7 +749,10 @@ int acg_setup_iso_0_9(int fd)
     }
 
   char *str="setup for rfdump";
-  return acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+  acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+
+  rfidReset(fd);
+  return 0;
 }
 
 /***************************************************************************/
@@ -860,7 +869,10 @@ int acg_setup_lfx_1_0(int fd)
     }
 
   char *str="setup for rfdump";
-  return acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+  acg_writeconfig(fd, (unsigned char*)str, 0xE0, 0xE0+strlen(str), NULL);
+
+  rfidReset(fd);
+  return 0;
 }
 
 int acg_printinfo_lfx_1_0(int fd)
