@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/callbacks.c,v 1.17 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/callbacks.c,v 1.21 2008/02/01 19:45:36 dirk Exp $ */
 
 #include <ctype.h>
 #include <gtk/gtk.h>
@@ -110,42 +110,103 @@ void on_about1_activate(GtkMenuItem *menuitem, gpointer user_data)
   gtk_widget_show(windowAbout);
 }
 
+static gboolean prefs_from_checkbox(char *id)
+{
+  gboolean b=FALSE;
+  GtkWidget *w=glade_xml_get_widget(glade_xml, id);
+  if(w)
+    {
+      b=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+      g_key_file_set_boolean(prefs, PREFS_GROUP, id, b);
+    }
+  else
+    fprintf(stderr, "could not find Widget %s\n", id);
 
-void on_buttonOK_clicked(GtkButton *button, gpointer user_data)
+  return b;
+}
+
+static const gchar* prefs_from_entry(char *id)
+{
+  const gchar *str=0;
+  GtkWidget *w=glade_xml_get_widget(glade_xml, id);
+  if(w)
+    {
+      str=gtk_entry_get_text(GTK_ENTRY(w));
+      g_key_file_set_value(prefs, PREFS_GROUP, id, str);
+    }
+  else
+    fprintf(stderr, "could not find Widget %s\n", id);
+
+  return str;
+}
+
+void on_preferencesSaved_clicked(GtkButton *button, gpointer user_data)
 {
   int i;
 
-  GtkWidget *cbCookieActive = glade_xml_get_widget(glade_xml, "cbCookieActive"); g_assert(cbCookieActive);
-  use_cookies = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbCookieActive))?TRUE:FALSE;
-  g_key_file_set_boolean(prefs, PREFS_GROUP, PREFS_COOKIE_ACTIVE, use_cookies);
+  use_cookies = prefs_from_checkbox(PREFS_COOKIE_ACTIVE);
 
-  GtkWidget *tfCookieID = glade_xml_get_widget(glade_xml, "tfCookieID"); g_assert(tfCookieID);
+  GtkWidget *tfCookieID = glade_xml_get_widget(glade_xml, PREFS_COOKIE); g_assert(tfCookieID);
   cookie_id = (gchar*)gtk_entry_get_text(GTK_ENTRY(tfCookieID));
   for(i=0; i<strlen(cookie_id); ++i)
     if(!isxdigit(cookie_id[i]))
       cookie_id[i]='0';
   g_key_file_set_value(prefs, PREFS_GROUP, PREFS_COOKIE, cookie_id);
 
-  GtkWidget *acg_serial_port=glade_xml_get_widget(glade_xml, "acg_serial_port"); g_assert(acg_serial_port);
-  const char *readerdevice=gtk_entry_get_text(GTK_ENTRY(acg_serial_port));
-  g_key_file_set_value(prefs, PREFS_GROUP, PREFS_SERIAL_PORT, readerdevice);
+  init_reader_arg a;
+  a.readerdevice = prefs_from_entry(PREFS_ACG_SERIALPORT);
+  a.baudrate = prefs_from_entry(PREFS_ACG_BAUDRATE);
+  a.binarymode = prefs_from_checkbox(PREFS_ACG_BINARYMODE);
 
+  writePreferences();
   gtk_widget_hide(windowPreferences);
+  poll_gtk();
+
+  if(a.readerdevice && a.baudrate)
+    init_reader(&a);
 }
 
 
+void on_preferencesCancel_clicked(GtkButton *button, gpointer user_data)
+{
+  gtk_widget_hide(windowPreferences);
+}
+
+static void entry_from_prefs(char *id)
+{
+  char *str=g_key_file_get_value(prefs, PREFS_GROUP, id, NULL);
+  if(str)
+    {
+      GtkWidget *w=glade_xml_get_widget(glade_xml, id);
+      if(w)
+	gtk_entry_set_text(GTK_ENTRY(w), str);
+      else
+	fprintf(stderr, "could not find Widget %s\n", id);
+    }
+}
+
+static void checkbox_from_prefs(char *id)
+{
+  gboolean b=g_key_file_get_boolean(prefs, PREFS_GROUP, id, NULL);
+  GtkWidget *w=glade_xml_get_widget(glade_xml, id);
+  if(w)
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), b);
+  else
+    fprintf(stderr, "could not find Widget %s\n", id);
+}
+
 void on_preferences1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-  GtkWidget *cbCookieActive = glade_xml_get_widget(glade_xml, "cbCookieActive"); g_assert(cbCookieActive);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbCookieActive), use_cookies?TRUE:FALSE);
+  checkbox_from_prefs(PREFS_COOKIE_ACTIVE);
 
+  entry_from_prefs(PREFS_COOKIE);
+#if 0
   GtkWidget *tfCookieID = glade_xml_get_widget(glade_xml, "tfCookieID"); g_assert(tfCookieID);
   gtk_entry_set_text(GTK_ENTRY(tfCookieID), cookie_id);
-
-  GtkWidget *acg_serial_port=glade_xml_get_widget(glade_xml, "acg_serial_port"); g_assert(acg_serial_port);
-  char *readerdevice=g_key_file_get_value(prefs, PREFS_GROUP, PREFS_SERIAL_PORT, NULL);
-  if(readerdevice)
-    gtk_entry_set_text(GTK_ENTRY(acg_serial_port), readerdevice);
+#endif
+  entry_from_prefs(PREFS_ACG_SERIALPORT);
+  entry_from_prefs(PREFS_ACG_BAUDRATE);
+  checkbox_from_prefs(PREFS_ACG_BINARYMODE);
 
   gtk_widget_show(windowPreferences);
 }
@@ -169,7 +230,7 @@ void on_stop_scan1_activate(GtkMenuItem *menuitem, gpointer user_data)
 
 void on_clear1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
-  clear_tag(&tag);
+  tag_clear(&tag);
   gtk_entry_set_text((GtkEntry*)entryTagID, "");
   gtk_entry_set_text((GtkEntry*)entryTagType, "");
   gtk_entry_set_text((GtkEntry*)entryManufacturer, "");
@@ -217,7 +278,17 @@ void on_set_to_defaults1_activate(GtkMenuItem *menuitem, gpointer user_data)
 void on_search_baud_rate1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
   stop_scan();
-  search_baud_rate(NULL);
+  int br=search_baud_rate();
+  if(br>0)
+    {
+      // TODO: save baudrate to preferences
+
+      char buf[128];
+      snprintf(buf, sizeof(buf), "Found Reader with %i baud", br);
+      info_dialog(buf);
+
+      start_scan();
+    }
 }
 
 void on_show_settings1_activate(GtkMenuItem *menuitem, gpointer user_data)
@@ -226,8 +297,3 @@ void on_show_settings1_activate(GtkMenuItem *menuitem, gpointer user_data)
   if(rfidReaderInfo)
     rfidReaderInfo(fd);
 }
-
-void on_binary_mode1_activate(GtkMenuItem *menuitem, gpointer user_data)
-{
-  binary_mode=!binary_mode;
-}
