@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/main.h,v 1.15 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/main.h,v 1.19 2008/02/01 19:45:36 dirk Exp $ */
 
 #ifndef MAIN__H
 #define MAIN__H
@@ -23,17 +23,29 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 #include <glade/glade.h>
 #include "rfid.h"
 
+char dec2hex(const int i);
+int getCharValue(char c);
+
 void stop_scan();
 void start_scan();
 void statusbar_set(char *str);
-int getCharValue(char c);
 void poll_gtk();
 void read_memory_tick(int);
 void write_memory_tick(int);
 void setup_table();
 void error_dialog(char *message);
+void info_dialog(char *message);
 
 void on_set_to_defaults1_activate(GtkMenuItem *menuitem, gpointer user_data);
+void writePreferences();
+
+typedef struct {
+  const char *readerdevice;
+  const char *baudrate;
+  int binarymode;
+} init_reader_arg;
+
+gint init_reader(gpointer data);
 
 extern int scan_run;
 extern int use_cookies;
@@ -48,8 +60,11 @@ extern GtkTreeModel *model;
 
 extern char *PREFS_FN;
 #define PREFS_GROUP "rfdump"
-#define PREFS_COOKIE "cookie"
+#define PREFS_COOKIE "cookie_id"
 #define PREFS_COOKIE_ACTIVE "cookie_active"
-#define PREFS_SERIAL_PORT "serial_port"
+
+#define PREFS_ACG_SERIALPORT "acg_serialport"
+#define PREFS_ACG_BAUDRATE "acg_baudrate"
+#define PREFS_ACG_BINARYMODE "acg_binarymode"
 
 #endif
