@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/rfid.c,v 1.43 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/rfid.c,v 1.54 2008/02/02 01:58:41 dirk Exp $ */
 
 #include <ctype.h>
 #include <errno.h>
@@ -34,10 +34,12 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 #include "rfid.h"
 #include "tools.h"
 
+#ifndef IUCLC
+#define IUCLC 0
+#endif
+
 int tcp_connect(const char *host, const char *port);
 
-char* RFID_CMD_RESET="x";
-char* RFID_CMD_SELECT="s";
 char* RFID_CMD_READ_PAGE="r";
 char* RFID_CMD_WRITE_PAGE="w";
 char* RFID_CMD_LOGIN;
@@ -61,32 +63,78 @@ static int baud_rate[] = {
     B1800,   1800,
     B2400,   2400,
     B4800,   4800,
+#ifdef B7200
+    B7200,   7200,
 #endif
+
+#endif
+
     B9600,   9600,
+#ifdef B14400
+    B14400,  14400,
+#endif
     B19200,  19200,
+#ifdef B28800
+    B28800,  28800,
+#endif
     B38400,  38400,
+#ifdef B57600
     B57600,  57600,
+#endif
+#ifdef B76800
+    B76800,  76800,
+#endif
+#ifdef B115200
     B115200, 115200,
+#endif
+#ifdef B230400
     B230400, 230400,
+#endif
+#ifdef B307200
+    B307200, 307200,
+#endif
+#ifdef B460800
     B460800, 460800,
-#if 0
+#endif
+#ifdef B500000
     B500000, 500000,
+#endif
+#ifdef B576000
     B576000, 576000,
+#endif
+#ifdef B921600
     B921600, 921600,
+#endif
+#ifdef B1000000
     B1000000,1000000,
+#endif
+#ifdef B1152000
     B1152000,1152000,
+#endif
+#ifdef B1500000
     B1500000,1500000,
+#endif
+#ifdef B2000000
     B2000000,2000000,
+#endif
+#ifdef B2500000
     B2500000,2500000,
+#endif
+#ifdef B3000000
     B3000000,3000000,
+#endif
+#ifdef B3500000
     B3500000,3500000,
+#endif
+#ifdef B4000000
     B4000000,4000000,
 #endif
   };
 
+static char* current_readerdevice;
+static int current_baudrate;
 static char lck[PATH_MAX];
-static struct termios oldt, curt;
-static char *readerdevice;
+static struct termios oldt;
 void serial_cleanup()
 {
   if(isatty(fd))
@@ -105,13 +153,16 @@ void serial_cleanup()
   if(lck[0])
     unlink(lck);
   lck[0]=0;
+
+  if(current_readerdevice)
+    {
+      free(current_readerdevice);
+      current_readerdevice=0;
+    }
 }
 
-int rfidConnectReader(char *readerdevice_, int baudrate)
+int rfidConnectReader(const char *readerdevice, int baudrate, int binarymode_)
 {
-  if(readerdevice_)
-    readerdevice=readerdevice_;
-
   g_assert(readerdevice);
 
   serial_cleanup();
@@ -126,6 +177,7 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
       }
   }
 
+#if defined(__linux__)
   /* check lockfile */
   {
     int pos=strlen(readerdevice)-1;
@@ -135,6 +187,7 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
 	  break;
 	--pos;
       }
+
     SNPRINTF(lck, sizeof(lck), "/var/lock/LCK..%s", readerdevice+pos+1);
     if(access(lck, F_OK) == 0)
       {
@@ -156,10 +209,14 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
     fprintf(f, "%i rfdump %s\n", getpid(), user);
     fclose(f);
   }
+#endif
 
   atexit(serial_cleanup);
 
   /* open */
+#if B2400==2400 && B9600==9600 && B38400==38400
+  int br=baudrate;
+#else
   int br=-1, i;
   for(i=0; i<sizeof_array(baud_rate); i+=2)
     if(baud_rate[i]==baudrate)
@@ -182,6 +239,7 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
       fprintf(stderr, "unknown baud rate %i == 0x%08X\n", baudrate, baudrate);
       return -1;
     }
+#endif
   fprintf(stderr, "using serial device %s %i baud\n", readerdevice, br);
 
   fd = open(readerdevice, O_RDWR | O_NOCTTY);
@@ -193,6 +251,7 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
 
   tcflush(fd, TCIOFLUSH);
 
+  struct termios curt;
   if(tcgetattr(fd, &curt) < 0)
     {
       fprintf(stderr, "could not get serial device attributes: %s\n", strerror(errno));
@@ -208,9 +267,6 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
   curt.c_cflag &= ~CSIZE;
   curt.c_cflag |= CS8;
 
-  /* no software flow control */
-  curt.c_cflag &= ~(IXON|IXOFF);
-
   /* no hardware handshake */
   curt.c_cflag &= ~CRTSCTS;
   curt.c_cflag |= CLOCAL;
@@ -228,14 +284,17 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
   curt.c_iflag |= IGNPAR;	/* ignore parity errors */
   curt.c_iflag &= ~INPCK;	/* disable parity check */
 
-  /* no local flags */
-  curt.c_lflag = 0;
+  /* no software flow control */
+  curt.c_iflag &= ~(IXON|IXOFF);
 
   /* disable input processing */
   curt.c_iflag &= ~(INPCK|PARMRK|BRKINT|INLCR|ICRNL|IUCLC|IXANY);
   /* ignore break */
   curt.c_iflag |= IGNBRK;
 
+  /* no local flags */
+  curt.c_lflag = 0;
+
   curt.c_oflag=0;
 
   /* set read timeout = .1s */
@@ -251,6 +310,9 @@ int rfidConnectReader(char *readerdevice_, int baudrate)
 
   tcflush(fd, TCIOFLUSH);
 
+  current_readerdevice=strdup(readerdevice);
+  current_baudrate=br;
+  binary_mode=binarymode_;
   return fd;
 }
 
@@ -281,6 +343,7 @@ int rfidReadAllPages(int fd, struct RFIDTag *tag)
 	  fprintf(stderr, "could not read page %02X\n", adr);
 	  return -1;
 	}
+      usleep(10000);		/* some serial drivers make shit when used too fast... */
     }
 
   tag->memSize=adr;
@@ -351,7 +414,7 @@ int rfidReadLine(int fd, char *result, int bufsize)
   g_assert(result);
   g_assert(bufsize > 0);
 
-  char buf[bufsize+10];
+  unsigned char buf[bufsize+10];
   int pos = 0;
   int timeout=2;//10;		/* wait for 10s for on line */
 
@@ -375,27 +438,39 @@ int rfidReadLine(int fd, char *result, int bufsize)
 	{
 	  if(--timeout == 0)
 	    return -1;
-	  else
-	    poll_gtk();
+
+	  poll_gtk();
 	}
       else if(FD_ISSET(fd, &rfds))
 	{
-	  char inputChar;
+	  unsigned char inputChar;
 	  const int numRead = read(fd,&inputChar,1);
 
 	  if (numRead == 1)
 	    {
-	      //fprintf(stderr, "read %02X %c\n", inputChar, isprint(inputChar)?inputChar:'.');
-
+#if defined(DOJDEBUG) && 0
+	      fprintf(stderr, "read %02X %c\n", inputChar, isprint(inputChar)?inputChar:'.');
+#endif
 	      buf[pos] = inputChar;
 
-	      /* search for newline */
-	      if(buf[pos] == '\n')
+	      if(binary_mode)
 		{
-		  buf[pos]=0;	/* remove newline from result */
-		  if(pos>0 && buf[pos-1]=='\r')
-		    buf[pos-1]=0;	/* remove cr too */
-		  break;
+		  if(buf[pos]==3) /* ETX */
+		    {
+		      ++pos;
+		      break;
+		    }
+		}
+	      else
+		{
+		  /* search for newline */
+		  if(buf[pos] == '\n')
+		    {
+		      buf[pos]=0;	/* remove newline from result */
+		      if(pos>0 && buf[pos-1]=='\r')
+			buf[pos-1]=0;	/* remove cr too */
+		      break;
+		    }
 		}
 
 	      buf[++pos] = 0;	/* set end-of-string */
@@ -411,15 +486,50 @@ int rfidReadLine(int fd, char *result, int bufsize)
   //fprintf(stderr, "rfidReadLine:%s\n", buf);
 
   /* check for binary frame */
-  int len=strlen(buf);
-  if(buf[0]==2 && buf[len-1]==3)
+  if(binary_mode)
     {
-      /* TODO: check BCC */
-      strncpy(result, buf+3, len-5);
-      result[len-5]=0;
+      if(pos < 6)
+	{
+	  fprintf(stderr, "Received frame too short: %i bytes\n", pos);
+	  return -1;
+	}
+
+      int framelen=buf[2];
+      if(framelen==0) framelen=256;
+      if(framelen != pos-5)
+	{
+	  fprintf(stderr, "Invalid Framelength: %i!=%i\n", framelen, pos-5);
+	  return -1;
+	}
+
+      if(buf[0]==2 && buf[pos-1]==3)
+	{
+	  unsigned char bcc=0;
+	  int i;
+	  for(i=1; i<pos-2; ++i)
+	    bcc^=buf[i];
+	  if(buf[pos-2] != bcc)
+	    {
+	      fprintf(stderr, "Checksum error on result frame %02X!=%02X\n", buf[pos-2], bcc);
+	      return -1;
+	    }
+	  memcpy(result, buf+3, pos-5);
+	  result[pos-5]=0;
+	  return pos-5;
+	}
+      else
+	{
+	  fprintf(stderr, "Invalid Framebytes %02X %02X\n", buf[0], buf[pos-1]);
+	  return -1;
+	}
+
     }
   else
-    strcpy(result, buf);
+    {
+      strcpy(result, (char*)buf);
+      return strlen(result);
+    }
+
   return 0;
 }
 
@@ -435,6 +545,7 @@ static int readpage_std(int fd, int pageNo, char *page, int pagesize)
   char rfidCmd[255];
 
   /* write command to card reader */
+  tcflush(fd, TCIOFLUSH);
   g_assert(RFID_CMD_READ_PAGE);
   SNPRINTF(rfidCmd, sizeof(rfidCmd), "%s%02X", RFID_CMD_READ_PAGE, pageNo);
   if(rfidWrite(fd, rfidCmd) < 0)
@@ -444,8 +555,8 @@ static int readpage_std(int fd, int pageNo, char *page, int pagesize)
   if(rfidReadLine(fd, page, pagesize) < 0)
     return -1;
 
-  //fprintf(stderr, "readpage_std(%02X):%s\n", pageNo, page);
-
+  //fprintf(stderr, "readpage_std(%02X):%s\n", pageNo, page)
+;
   const int len=strlen(page);
   if(len <= 0)
     {
@@ -491,6 +602,7 @@ static int writepage_std(int fd, int pageNo, char *page)
   g_assert(fd >= 0);
 
   /* send cmd to reader */
+  tcflush(fd, TCIOFLUSH);
   char rfidCmd[255];
   g_assert(RFID_CMD_WRITE_PAGE);
   SNPRINTF(rfidCmd, sizeof(rfidCmd), "%s%02X%s", RFID_CMD_WRITE_PAGE, pageNo, page);
@@ -704,14 +816,17 @@ static int writepage_mifare(int fd, int pageNo, char *page)
   return 0;
 }
 
-void clear_tag(struct RFIDTag *t)
+void tag_clear(struct RFIDTag *t)
 {
+  int i;
+
   g_assert(t);
 
   t->tagID[0] = '\0';
   t->tagTypeName[0] = '\0';
   t->tagManufacturerName[0] = '\0';
-  int i;
+  t->memSize=1;
+  t->pageSize=4;
   for(i=0; i<sizeof_array(t->mem); ++i)
     {
       g_free(t->mem[i]);
@@ -729,22 +844,116 @@ void clear_tag(struct RFIDTag *t)
     }
 }
 
+void tag_copy(struct RFIDTag *dst, const struct RFIDTag *src)
+{
+  int i;
+
+  g_assert(src);
+  g_assert(dst);
+
+  g_strlcpy(dst->tagID, src->tagID, sizeof_array(dst->tagID));
+  g_strlcpy(dst->tagTypeName, src->tagTypeName, sizeof_array(dst->tagTypeName));
+  g_strlcpy(dst->tagManufacturerName, src->tagManufacturerName, sizeof_array(dst->tagManufacturerName));
+  dst->memSize=src->memSize;
+  dst->pageSize=src->pageSize;
+  for(i=0; i<sizeof_array(dst->mem); ++i)
+    {
+      g_free(dst->mem[i]);
+      dst->mem[i]=g_strdup(src->mem[i]);
+    }
+  for(i=0; i<sizeof_array(dst->keya); ++i)
+    {
+      g_free(dst->keya[i]);
+      dst->keya[i]=g_strdup(src->keya[i]);
+    }
+  for(i=0; i<sizeof_array(dst->keyb); ++i)
+    {
+      g_free(dst->keyb[i]);
+      dst->keyb[i]=g_strdup(src->keyb[i]);
+    }
+}
+
+uint8_t tag_getbyte(const struct RFIDTag *t, const unsigned n)
+{
+  g_assert(t);
+  if(n>=t->memSize*t->pageSize)
+    return 0;
+  const unsigned page=n / t->pageSize;
+  const unsigned i=n % t->pageSize;
+  if(! t->mem[page])
+    return 0;
+  if(strlen(t->mem[page]) < i*2)
+    return 0;
+  const uint8_t b=getCharValue(t->mem[page][i*2])*16 + getCharValue(t->mem[page][i*2+1]);
+#if 0
+  fprintf(stderr, "tag_getbyte(,%i)=%02X %c\n", n, b, isprint(b)?b:'.');
+#endif
+  return b;
+}
+
+int tag_setbyte(struct RFIDTag *t, const unsigned n, const uint8_t b)
+{
+  g_assert(t);
+  if(n>=t->memSize*t->pageSize)
+    return -1;
+  const unsigned page=n / t->pageSize;
+  const unsigned i=n % t->pageSize;
+  if(! t->mem[page])
+    {
+      t->mem[page]=g_strnfill(t->pageSize*2, '0');
+      g_assert(t->mem[page]);
+    }
+  else if(strlen(t->mem[page]) < t->pageSize*2)
+    {
+      char *s=g_strndup(t->mem[page], t->pageSize*2);
+      g_assert(s);
+      g_free(t->mem[page]);
+      t->mem[page]=s;
+    }
+  t->mem[page][i*2]=dec2hex(b>>4);
+  t->mem[page][i*2+1]=dec2hex(b&15);
+
+  return b;
+}
+
+void tag_copydata(struct RFIDTag *dst, const struct RFIDTag *src)
+{
+  int i;
+
+  g_assert(src);
+  g_assert(dst);
+
+  for(i=0; i<src->memSize*src->pageSize && i<dst->memSize*dst->pageSize; ++i)
+    tag_setbyte(dst, i, tag_getbyte(src, i));
+
+  for(i=0; i<sizeof_array(dst->keya); ++i)
+    {
+      g_free(dst->keya[i]);
+      dst->keya[i]=g_strdup(src->keya[i]);
+    }
+  for(i=0; i<sizeof_array(dst->keyb); ++i)
+    {
+      g_free(dst->keyb[i]);
+      dst->keyb[i]=g_strdup(src->keyb[i]);
+    }
+}
+
 int rfidReset(int fd)
 {
   g_assert(fd >= 0);
-  g_assert(RFID_CMD_RESET);
 
-  clear_tag(&tag);
+  tag_clear(&tag);
 
   /* reset reader */
+  tcflush(fd, TCIOFLUSH);
   char rfidCmd[255];
-  int try;
-  for(try=0; try<2; ++try)
+  int attempt;
+  for(attempt=0; attempt<2; ++attempt)
     {
       fprintf(stderr, "RESET\n");
 
-      rfidCmd[0] = '\0';
-      strcat(rfidCmd, RFID_CMD_RESET);
+      rfidCmd[0] = binary_mode?'v':'x';
+      rfidCmd[1] = '\0';
 
       if(rfidWrite(fd, rfidCmd) < 0)
 	return -1;
@@ -775,7 +984,7 @@ int rfidReset(int fd)
     {
       rfidReaderSetup=acg_setup_iso_0_9;
       rfidReaderInfo=acg_printinfo_iso_0_9;
-      return 0;
+      goto reader_found;
     }
   if(!strcmp(rfidCmd, "Dual 2.1") || !strcmp(rfidCmd, "Dual 2.2") || !strcmp(rfidCmd, "MultiISO 1.0"))
     {
@@ -792,7 +1001,7 @@ int rfidReset(int fd)
 	  rfidReaderSetup=acg_setup_dual_2_2;
 	  rfidReaderInfo=acg_printinfo_dual_2_2;
 	}
-      return 0;
+      goto reader_found;
     }
   if(!strcmp(rfidCmd, "LFX 1.0"))
     {
@@ -801,11 +1010,26 @@ int rfidReset(int fd)
       RFID_CMD_LOGIN="l";
       rfidReaderSetup=acg_setup_lfx_1_0;
       rfidReaderInfo=acg_printinfo_lfx_1_0;
-      return 0;
+      goto reader_found;
     }
 
   fprintf(stderr, "unknown reader\n");
   return -1;
+
+ reader_found:
+
+  /* disable continuous select */
+  if(!binary_mode)
+    {
+      rfidCmd[0] = '.';
+      rfidCmd[1] = '\0';
+      rfidWrite(fd, rfidCmd);
+      usleep(500000);
+    }
+
+  tcflush(fd, TCIOFLUSH);
+
+  return 0;
 }
 
 static void mifare_set_key(char *key, const char *s)
@@ -848,39 +1072,52 @@ int rfidWrite(int fd, char *cmd)
   g_assert(fd >= 0);
   g_assert(cmd);
 
-  //fprintf(stderr, "rfidWrite:%s\n", cmd);
-
   int len=strlen(cmd);
+  if(len<=0)
+    {
+      fprintf(stderr, "rfidWrite(): empty command\n");
+      return -1;
+    }
   const int maxlen=255;
   if(len>maxlen)
     {
-      fprintf(stderr, "length of cmd '%s' is %i chars and too long.\n", cmd, len);
+      fprintf(stderr, "rfidWrite(): length of cmd '%s' is %i chars and too long.\n", cmd, len);
       return -1;
     }
 
-  char buf[maxlen+5+1];
+  unsigned char buf[maxlen+5+1];
   if(binary_mode)
     {
       buf[0]=2;			/* STX */
       buf[1]=0xFF;		/* Station ID, 0xFF=all stations */
       buf[2]=len;
       memcpy(buf+3, cmd, len);	/* copy cmd */
+
       /* calculate BCC checksum */
-      unsigned char bcc=0, *data=(unsigned char*)cmd;
+      unsigned char bcc=buf[1];
       int i;
-      for(i=0; i<len; ++i)
-	bcc^=data[i];
+      for(i=0; i<len+1; ++i)
+	bcc^=buf[i+2];
       buf[3+len]=bcc;
+
       buf[3+len+1]=3;		/* ETX */
-      buf[3+len+2]=0;
+      buf[3+len+2]=0;		/* c-string termination */
       len+=5;
-      cmd=buf;
+      cmd=(char*)buf;
     }
 
+#if defined(DOJDEBUG) && 0
+  fprintf(stderr, "rfidWrite:%s:", cmd);
+  int i;
+  for(i=0; i<len; ++i)
+    fprintf(stderr, " %02X", ((unsigned char*)cmd)[i]);
+  fprintf(stderr, "\n");
+#endif
+
   int w=0;
   while(len>0)
     {
-      int W=write(fd, cmd+w, len);
+      const int W=write(fd, cmd+w, len);
       if(W>0)
 	{
 	  w+=W;
@@ -901,32 +1138,60 @@ int rfidSelect(int fd, char *result, int result_size)
   g_assert(result);
   g_assert(result_size > 0);
 
-  g_assert(RFID_CMD_SELECT);
-  write(fd, RFID_CMD_SELECT, 1);
+  rfidWrite(fd, "s");
+
+  if(binary_mode)
+    {
+      int r=rfidReadLine(fd, result, result_size);
+      if(r<=1) return r;
+
+      unsigned char buf[result_size*2+10], *res=(unsigned char*)result;
+      buf[0]=result[0];
+      int s,d;
+      for(s=1,d=1; s<r; ++s)
+	{
+	  buf[d++]=dec2hex(res[s]>>4);
+	  buf[d++]=dec2hex(res[s]&0xF);
+	}
+      buf[d]=0;
+
+      strcpy(result, (char*)buf);
+      return strlen(result);
+    }
+
   return rfidReadLine(fd, result, result_size);
 }
 
-int search_baud_rate(char *readerdevice)
+int search_baud_rate()
 {
-  g_assert(RFID_CMD_RESET);
+  if(!current_readerdevice)
+    {
+      fprintf(stderr, "don't know which reader device to use\n");
+      return -1;
+    }
 
-  int i;
+  char *readerdevice=strdup(current_readerdevice);
+
+  int i,ret=-1;
   for(i=0; i<sizeof_array(baud_rate); i+=2)
     {
       fprintf(stderr, "trying %i baud\n", baud_rate[i+1]);
 
-      fd = rfidConnectReader(readerdevice, baud_rate[i]);
+      fd = rfidConnectReader(readerdevice, baud_rate[i], binary_mode);
       if(fd < 0)
 	{
 	  fprintf(stderr, "could not open serial port and set to %i baud\n", baud_rate[i+1]);
-	  return -1;
+	  goto finish;
 	}
       if(rfidReset(fd) == 0)
 	{
 	  fprintf(stderr, "found reader\n");
-	  return 0;
+	  ret=current_baudrate;
+	  goto finish;
 	}
     }
 
-  return -1;
+ finish:
+  free(readerdevice);
+  return ret;
 }
