@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,11 +14,13 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/rfid.h,v 1.19 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/rfid.h,v 1.23 2008/02/02 01:58:41 dirk Exp $ */
 
 #ifndef RFID__H
 #define RFID__H
 
+#include <stdint.h>
+
 #define RFID_SIZE 256
 struct RFIDTag {
   char tagID[RFID_SIZE];
@@ -32,10 +34,14 @@ struct RFIDTag {
 };
 #undef RFID_SIZE
 
+void tag_clear(struct RFIDTag *t);
+void tag_copy(struct RFIDTag *dst, const struct RFIDTag *src);
+void tag_copydata(struct RFIDTag *dst, const struct RFIDTag *src);
+uint8_t tag_getbyte(const struct RFIDTag *t, const unsigned n);
+int tag_setbyte(struct RFIDTag *t, const unsigned n, const uint8_t b);
+
 /* RFID API constants */
 
-extern char* RFID_CMD_RESET;
-extern char* RFID_CMD_SELECT;
 extern char* RFID_CMD_READ_PAGE;
 extern char* RFID_CMD_WRITE_PAGE;
 extern char* RFID_CMD_LOGIN;
@@ -67,9 +73,7 @@ extern char* RFID_CMD_LOGIN;
 
 extern int binary_mode;
 
-void clear_tag(struct RFIDTag*);
-
-int rfidConnectReader(char *readerdevice, int baudrate);
+int rfidConnectReader(const char *readerdevice, int baudrate, int binarymode_);
 int rfidReset(int fd);
 void serial_cleanup();
 
@@ -92,6 +96,6 @@ int rfidWrite(int fd, char *cmd);
 extern int (*rfidReaderSetup)(int fd);
 extern int (*rfidReaderInfo)(int fd);
 
-int search_baud_rate(char *readerdevice);
+int search_baud_rate();
 
 #endif
