@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/xml.c,v 1.12 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/xml.c,v 1.13 2008/01/23 00:51:50 dirk Exp $ */
 
 #include <ctype.h>
 #include <errno.h>
@@ -26,10 +26,13 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 #include "tools.h"
 #include "xml.h"
 
-int xmlWriteTag(struct RFIDTag *tag, char *fn)
+int xmlWriteTag(const struct RFIDTag *tag, const char *fn)
 {
   FILE *fp;
 
+  g_assert(tag);
+  g_assert(fn);
+
   if ((fp=fopen(fn,"w")) == NULL)
     {
       fprintf(stderr, "Could not open %s: %s\n", fn, strerror(errno));
@@ -38,7 +41,7 @@ int xmlWriteTag(struct RFIDTag *tag, char *fn)
 
   fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
   fprintf(fp, "<!DOCTYPE tag SYSTEM \"rfd.dtd\">\n");
-  fprintf(fp, "<rfd:tag xmlns:rfd=\"http://www.rfdump.org/rfd\">\n");
+  fprintf(fp, "<rfd:tag xmlns:rfd=\"http://www.rfdump.org/rfd\" version=\"1.0\">\n");
   fprintf(fp, "  <rfd:tagID>%s</rfd:tagID>\n", tag->tagID);
   fprintf(fp, "  <rfd:tagTypeName>%s</rfd:tagTypeName>\n", tag->tagTypeName);
   fprintf(fp, "  <rfd:tagManufacturerName>%s</rfd:tagManufacturerName>\n", tag->tagManufacturerName);
@@ -63,7 +66,7 @@ int xmlWriteTag(struct RFIDTag *tag, char *fn)
 	    if(*p)
 	      {
 		ascii+=getCharValue(*p++);
-		fprintf(fp, "%c", (isprint(ascii) && ascii<128)?ascii:'.');
+		fputc((isprint(ascii) && ascii!='>' && ascii<128)?ascii:'.', fp);
 	      }
 	  }
 	fprintf(fp, " -->\n");
@@ -83,14 +86,18 @@ static int depth;
 static int offset;
 static int offset_max;
 static const char *element;
-static char *mifare_keya, *mifare_keyb;
-static struct RFIDTag *tmpTag;
+static const char *mifare_keya, *mifare_keyb;
+static struct RFIDTag tmpTag;
 
 #define XMLDEBUG 0
 
 static void XMLCALL start(void *data, const char *el, const char **attr)
 {
   int i;
+
+  g_assert(el);
+  g_assert(attr);
+
 #if XMLDEBUG
   for (i = 0; i < depth; i++)
     printf("  ");
@@ -111,14 +118,14 @@ static void XMLCALL start(void *data, const char *el, const char **attr)
 	  if (!strcmp(attr[i], "offset"))
 	    offset=strtoul(attr[i+1], 0, 16);
 	  else if(!strcmp(attr[i], "mifare_keya"))
-	    mifare_keya=g_strdup(attr[i+1]);
+	    mifare_keya=attr[i+1];
 	  else if(!strcmp(attr[i], "mifare_keyb"))
-	    mifare_keyb=g_strdup(attr[i+1]);
+	    mifare_keyb=attr[i+1];
 	}
       else if(!strcmp(element, "rfd:tagData"))
 	{
 	  if (!strcmp(attr[i], "pagesize"))
-	    tmpTag->pageSize=strtoul(attr[i+1], 0, 16);
+	    tmpTag.pageSize=strtoul(attr[i+1], 0, 16);
 	}
     }
 
@@ -131,6 +138,8 @@ static void XMLCALL start(void *data, const char *el, const char **attr)
 
 static void XMLCALL content(void *data, const char *s, int len)
 {
+  g_assert(s);
+
   char *textinfo=g_strndup(s, len);
   /* strip whitespace */
   g_strstrip(textinfo);
@@ -167,39 +176,39 @@ static void XMLCALL content(void *data, const char *s, int len)
 	    return;
 	  }
 
-      if(offset>=0 && offset < sizeof_array(tmpTag->mem))
+      if(offset>=0 && offset < sizeof_array(tmpTag.mem))
 	{
-	  tmpTag->mem[offset]=g_strdup(textinfo);
+	  tmpTag.mem[offset]=g_strdup(textinfo);
 	  if(mifare_keya)
-	    tmpTag->keya[offset]=mifare_keya;
+	    tmpTag.keya[offset]=g_strdup(mifare_keya);
 	  if(mifare_keyb)
-	    tmpTag->keyb[offset]=mifare_keyb;
+	    tmpTag.keyb[offset]=g_strdup(mifare_keyb);
 
 	  if(offset>offset_max)
 	    offset_max=offset;
 	}
       else
 	{
-	  fprintf(stderr, "offset attribute exceeds tag storate: %02X >= %02X", offset, sizeof_array(tmpTag->mem));
+	  fprintf(stderr, "offset attribute exceeds tag storate: %02X >= %02X", offset, sizeof_array(tmpTag.mem));
 	}
     }
   else if(!strcmp(element, "rfd:tagID"))
     {
       element="";
-      tmpTag->tagID[0]=0;
-      strcat(tmpTag->tagID, textinfo);
+      tmpTag.tagID[0]=0;
+      strcat(tmpTag.tagID, textinfo);
     }
   else if(!strcmp(element, "rfd:tagTypeName"))
     {
       element="";
-      tmpTag->tagTypeName[0]=0;
-      strcat(tmpTag->tagTypeName, textinfo);
+      tmpTag.tagTypeName[0]=0;
+      strcat(tmpTag.tagTypeName, textinfo);
     }
   else if(!strcmp(element, "rfd:tagManufacturerName"))
     {
       element="";
-      tmpTag->tagManufacturerName[0]=0;
-      strcat(tmpTag->tagManufacturerName, textinfo);
+      tmpTag.tagManufacturerName[0]=0;
+      strcat(tmpTag.tagManufacturerName, textinfo);
     }
   g_free(textinfo);
 }
@@ -209,23 +218,12 @@ static void XMLCALL end(void *data, const char *el)
   depth--;
 }
 
-int xmlReadTag(struct RFIDTag *tag, char *fn)
+int xmlReadTag(struct RFIDTag *tag, const char *fn)
 {
-  clear_tag(tag);
-  offset_max=0;
-
-  tmpTag = tag;
-
-  XML_Parser p = XML_ParserCreate(NULL);
-  if (!p)
-    {
-      fprintf(stderr, "Couldn't allocate memory for parser\n");
-      return -1;
-    }
-
-  XML_SetElementHandler(p, start, end);
-  XML_SetCharacterDataHandler(p, content);
+  g_assert(tag);
+  g_assert(fn);
 
+  /* read fn into a memory buffer */
   FILE *fp;
   if ((fp=fopen(fn,"r")) == NULL)
     {
@@ -266,15 +264,37 @@ int xmlReadTag(struct RFIDTag *tag, char *fn)
 
   fclose(fp);
 
+  /* prepare xml reading environment */
+  tag_clear(&tmpTag);
+  depth=offset=offset_max=0;
+  element=0;
+
+  /* create xml parser and parse the xml document */
+  XML_Parser p = XML_ParserCreate(NULL);
+  if (!p)
+    {
+      fprintf(stderr, "Couldn't allocate memory for parser\n");
+      g_free(buff);
+      return -1;
+    }
+
+  XML_SetElementHandler(p, start, end);
+  XML_SetCharacterDataHandler(p, content);
+
   int done=0;
   if (XML_Parse(p, buff, len, done) == XML_STATUS_ERROR)
     {
       fprintf(stderr, "Parse error at line %d:\n%s\n", (int)XML_GetCurrentLineNumber(p), XML_ErrorString(XML_GetErrorCode(p)));
-      return -1;
+      done=-1;
+    }
+  else
+    {
+      tmpTag.memSize=offset_max+1;
+      tag_copydata(tag, &tmpTag);
+      done=0;
     }
 
-  tmpTag->memSize=offset_max+1;
-
   g_free(buff);
-  return 0;
+  XML_ParserFree(p);
+  return done;
 }
