@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2007 DN-Systems GmbH http://rfdump.org
+/* Copyright (c) 2005-2008 DN-Systems GmbH http://rfdump.org
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,14 +14,14 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
-/* $Header: /anoncvs/rfdump/src/xml.h,v 1.5 2007/10/31 21:59:27 dirk Exp $ */
+/* $Header: /cvs/rfdump/src/xml.h,v 1.6 2008/01/23 00:51:50 dirk Exp $ */
 
 #ifndef XML__H
 #define XML__H
 
 #include "rfid.h"
 
-int xmlWriteTag(struct RFIDTag *tag, char *fn);
-int xmlReadTag(struct RFIDTag *tag, char *fn);
+int xmlWriteTag(const struct RFIDTag *tag, const char *fn);
+int xmlReadTag(struct RFIDTag *tag, const char *fn);
 
 #endif
